#!/bin/bash
# 
# This scripts generates the input files for a ligand-receptor systems  
# 
# The script must be launched after the autopfs model generation with vmd (only protein) 
# and the primadorac processing of the ligand PDB file extracted MANUALLY from raw pdb 
# the ligand pdb must be named "lig.pdb"
#
#
if [ ! -z $ORAC_HOME  ] ;  then
    if [ -d $ORAC_HOME ] ; then
	DIRS=$ORAC_HOME
    else
	echo " ** ERROR ***"
	echo " given  ORAC_HOME variable "  $ORAC_HOME " does not look like the ORAC home"
	echo " set variable ORAC_HOME and retry "  
	exit 
    fi
else    
    echo "ORAC_HOME not set" 
    if [ -d ~/ORAC/trunk/src  ]; then 
	DIRS=~/ORAC/trunk  # default ORAC mount point  ; 
    elif [ -d ~/ORAC/src   ]; then  
	DIRS=~/ORAC  # default ORAC mount point  ; 
    else
	echo " ORAC_HOME not set; ORAC dir not found" 
	echo " set ORAC_HOME and try again"
    fi
fi 

pull=0
clean=0
commit=0
erro="none"

# parse options
while getopts ":pca" opt; do 
    case $opt in 
	c) 
	    clean=1; 
	    ;;
	\?) 
	    echo "invalid option"
	    ;;
    esac
done

filename=${!#}

if [ -z $1 ]; then
    echo " " 
    echo " This scripts generates the ORAC input files for a ligand-receptor systems  "
    echo " Syntax: " 
    echo  $0 " [ -c ] pdbcode" 
    echo " " 
    echo  "    Args:" 
    echo  "         pdbcode is the four digits alphanumeric PDB code of the ligand-target system " 
    echo  "    Options:"
    echo  "         -c  " 
    echo  "          clean directory after execution (only ORAC input files are spared)"  
    exit
 fi
 
# functions

function seqgen {
    grep " CA " pdb$1_autopsf.pdb  | awk -v ll=$2 'BEGIN{nter="-h"; cter="-o"} { i++; if(NR==1) {printf "%4s%2s", tolower($4),nter} else{ printf "%4s", tolower($4)}; if(i==ll) {printf "\n"; i=0}}END{printf "-o\n"}' > seq.tmp
    if [ -s lig-p.pdb -a -s lig-p.tpg -a -s lig-p.prm ] ; then
	resname=`awk '{if($1=="RESIDUE") {print $2}}' lig-p.tpg`
	cat  $DIRS/etc/templates/0.in seq.tmp  $DIRS/etc/templates/1.in | sed "s/resname/$resname/g" | sed "s/PDBF/$1/g" | sed "s?ORAC_HOME?$ORAC_HOME?g" > $1.in
	sed "s/     1 /   -10 /g" lig-p.pdb > tmp.lig
	cat tmp.lig  pdb${filename}_autopsf.pdb > ${filename}.pdb
	cp lig-p.tpg ${filename}.tpg
	cp lig-p.prm ${filename}.prm
	echo $1.in ${filename}" tpg/prm "  "input files generated; running orac..."
    else
	echo ""
	echo  " *** ERROR " 
	echo  " primadorac generated ligand tpg/prn/pdb files not found. " 
	echo  " Save ligand coordinates from file " pdb$1.ent " in a file named 'lig.pdb'  and launch primadorac as "
	echo  " $ORAC_HOME/tools/primadorac.bash -gdp lig.pdb"  
	echo ""
	exit
    fi
}



# generated sequence
if [ -s pdb${filename}_autopsf.pdb ] ; then
    seqgen $filename 13
else
    echo  " *** ERROR " 
    echo  " file "  pdb${filename}_autopsf.pdb " not found" 
    echo  " use vmd function  Extensions -> Modeling ->  Automatic PSF Builder to generate the protein target file" 
    exit
fi


if [ -s lig-p.pdb -a -s lig-p.tpg -a -s lig-p.prm ] ; then
    sed "s/     1 /   -10 /g" lig-p.pdb > tmp.lig
    cat tmp.lig  pdb${filename}_autopsf.pdb > ${filename}.pdb
    cp lig-p.tpg ${filename}.tpg
    cp lig-p.prm ${filename}.prm
else
    echo  " *** ERROR " 
    echo  " primadorac generated ligand tpg/prn/pdb files not found. " 
    echo  " Save ligand coordinates from file " pdb$1.ent " in a file named 'lig.pdb'  and launch primadorac as "
    echo  " $ORAC_HOME/tools/primadorac.bash -gdp lig.pdb"  
    exit
fi 

~/ORAC/trunk/src/INTEL-FFTW-OMP/orac < ${filename}.in > tmp.out;
erro=`$DIRS/tools/scripts/check_orac_output.awk tmp.out`
# this is fixable cysh error 
echo " orac finished. Check for fixable errors"

if [ "$erro" == "e13" ] ; then 
    echo " fixing wrong final residue (13-12).."
    seqgen $filename 12
    ~/ORAC/trunk/src/INTEL-FFTW-OMP/orac < ${filename}.in > tmp.out;
    erro=`$DIRS/tools/scripts/check_orac_output.awk tmp.out`
fi

if [ "$erro" == "cysh" ] ; then
    echo " fixing cys-> cysh and re-launching orac..."
    sed "s/cys/cysh/g" ${filename}.in > tmp.in; mv tmp.in ${filename}.in
    ~/ORAC/trunk/src/INTEL-FFTW-OMP/orac < ${filename}.in > tmp.out;
fi
if [ "$erro" == "ot2" ] ; then
    echo " fixing ot2 -> oc2 h and re-launching orac..."
    sed "s/ot2/oc2/g" ${filename}.in > tmp.in; mv tmp.in ${filename}.in
    sed "s/ot1/oc1/g" ${filename}.in > tmp.in; mv tmp.in ${filename}.in
    ~/ORAC/trunk/src/INTEL-FFTW-OMP/orac < ${filename}.in > tmp.out;
fi

$DIRS/tools/scripts/check_disulphide.awk ${filename}.pdb | tac > tmp.disul

nlines=`wc -l tmp.disul | awk '{print $1}'`

if [ $nlines != "2" ] ; then 
    echo " cys-cys bonds may be presents; updating the input accordingly... " 
    ncys=`wc -l  tmp.disul | awk '{print $1}'` 
    for((i=1; i<=$ncys ; i++)) ; do
	rm 1.pdb >& /dev/null
	linecys="`head -$i tmp.disul | tail -1`"
        sed -i "s/#TPGCYS/#TPGCYS\n$linecys/g" ${filename}.in
	~/ORAC/trunk/src/INTEL-FFTW-OMP/orac < ${filename}.in > tmp.out;
    done
fi

if [ -s 1.pdb ] ; then
    echo " Everything seems OK !! 1.pdb is not empty."
else
    tail -20 tmp.out
    echo " 1.pdb is empty and no fixable errors found..";
    echo " unless there are cys-cys and cysh at the same     ";
    echo " time. In that case, you can try to pull anyway," ;
    echo " find the true cys-cys and fix it by hand in ";
    echo " the sequence specified in "${filename}.in ;
fi
 
if [ $clean ==  1 ] ; then
    for i in `ls *` ; do 
        if [[ "$i" != ${filename}.* ]]; then # clean everything except input files 
	    rm $i 
	fi
    done
fi
